clear all; 

%% cartpole parameters
m = 0.1;
L = 1;
g = 9.81;
M = 1;
d1 = 1;
d2 = 0.5;

% linearized system (theta = 0)
q_ = (m+M) * g / (M*L);
A = [0,1,0,0 ; 0,-d1, -g*m/M,0 ; 0,0,0,1 ; 0,d1/L,q_,-d2] ;
B = [0, 1.0/M, 0., -1/(M*L)].' ; 
C_angle = [0,0,1,0];
C_cart = [1,0,0,0];
D = 0;

%% Luenberger observer design
p = 2.5*[-10,-2, -5, -20];
C = [1,0,0,0;0,0,1,0];
K_luen = place(A.',C',p);

%% observation / process noise
q1 = 5e-4; 
q2 = 1e-3; 

r1 = 1e-3;
r2 = 5e-4;

% system initial condition
x0 = [0,0,0.5, .1];

% initial estimated process noise
P_k_init = .01;

%% optimal control design

% continuous time
R_ctrl = 10;
Q_ctrl = diag([10,5,20,10]);

[X,L,K_lqr] = care(A,B,Q_ctrl, R_ctrl);

% discrete time
Ts = 0.01;

Ad = expm(A*Ts);
Bd = pinv(A)*(Ad-eye(4))*B;

[X,L,K_lqr_d] = dare(Ad,Bd,Q_ctrl, R_ctrl);


%% frequency analysis (optional, not used)
[b_angle,a_angle] = ss2tf(A,B,C_angle,D);
[b_cart,a_cart] = ss2tf(A,B,C_cart,D);

sys_angle = tf(b_angle,a_angle);
sys_cart = tf(b_cart,a_cart);

%rlocus(sys_angle)
%rlocus(sys_cart)
